unit VertialField;

interface

uses
    Forms, SysUtils, Classes, Types;

//////
const
    //  e
    KIND_NUM = 7;
    //  tB[hTCY
    FIELD_SIZE = 8;
    //  Z
    NULL_CELL : Integer = 0;

    //  Xe[^X
    STATUS_NONE   : Byte = 0;
    STATUS_DELETE : Byte = 1;
    STATUS_PONG   : Byte = 2;
    STATUS_NOMOVE : Byte = 3;

type
    //  
    TDirection = (dNone, dLeft, dDown, dRight, dUp);

    //  tB[hTCY`
    TIntField = array[0..FIELD_SIZE - 1] of array[0..FIELD_SIZE - 1] of Integer;
    TBoolField = array[0..FIELD_SIZE - 1] of array[0..FIELD_SIZE - 1] of Boolean;

    //  ޕʃJE^z
    TIntCounter = array[1..KIND_NUM] of Integer;

    //  
    TMode = (
        mIdle,              //  ҂
        mChange,            //  
        mCheckViaChange,    //  ɂ`FbN
        mBack,              //  `FbNsŖ߂
        mDelete,            //  `FbNŏ
        mDropFirst,         //  ̃}X⊮(ƂĂ)
        mDrop,              //  2ڈȍ~̃}X⊮(ƂĂ)
        mCheck,             //  `FbN(mDelete smIdle)
        mCheckNoMove,       //  ss\`FbN
        mNoMove,            //  ss\̂ߑւ
        mNoMoved            //  ss\̂ߑւ
        );

//           <----------------+
//            |                     |
//            V                     |
//          mIdle <--------------+  |
//            |                  |  |
//            V                  |  |
//          mChange              |  |
//            |                  |  |
//            V                  |  |
//          mCheckViaChange      |  |
//            |T         |F      |  |
//            |          |       |  |
//            V          V       |  |
//   +----> mDelete    mBack-->--+  |
//   |        |                  |  |
//   |        V                  |  |
//   |      mDropFirst           |  |
//   |        |                  |  |
//   |        V                  |  |
//   |      mDrop <--+           |  |
//   |      |F  |T   |           |  |
//   |      |   +----+           |  |
//   |      V                    |  |
//   |     mCheck                |  |
//   |      |T  | F              |  |
//   +---<--+   |                |  |
//              |                |  |
//           mCheckNoMove        |  |
//             |T    |F          |  |
//             |     +-------->--+  |
//             |                    |
//             +----------------->--+

//-----------------------------------------------------------------------------
type
    TGameThread = class(TThread)

    private
        FSelect: TPoint;
        FNeighbor: TDirection;
        procedure ClearSelect();
        function CheckNeihbor(const APoint: TPoint): Boolean;
    public
        procedure SetPoint(const APoint: TPoint);
        property Select: TPoint read FSelect;
    private
        FMode: TMode;                   //  ݂̃[h
        procedure RepaintForm();        //  ʍXV

    private
        FLevel: Integer;                //  ݂̃x
        FThreshold: Integer;            //  ̃xւ̃NA|Cg
        FLevelTimer: Integer;           //  xJñ^C}[l
        FTimer: Integer;                //  c莞
        FScore: Integer;                //  XRA
        FLevelUpped: Boolean;           //  xAbv
        FBonusKind: Integer;            //  XRA2{̎
        FGameOvered: Boolean;           //  Q[I[o[ɂȂ
        procedure NextLevel();          //  xAbv

    private
        FChainCounter: Integer;         //  AJE^
        FDiffCounter: TIntCounter;      //  ƃJE^

        Flag: TBoolField;               //  `FbNσZL^tB[h
        procedure InitField();          //  tB[hNA

        procedure OnIdle();             //  u҂v̏

        procedure OnChange();           //  uv

        function Check(): Boolean;      //  u`FbNv
        procedure OnCheck();            //  u`FbNv
        procedure CheckLevel();         //  x

        procedure Delete();             //  uv
        procedure OnDelete();           //  uv

        function Drop(): Boolean;       //  uv
        procedure OnDrop();             //  uv

        function CheckNoMove(var APoint: TPoint; var ADirction: TDirection): Boolean;
                                        //  uss\`FbNv
        procedure OnCheckNoMove();      //  uss\`FbNv

        procedure OnNoMove();           //  uւv
        procedure AfterNoMove();        //  uւv

        procedure ClearGrid();          //  ւ̂߂̃tO

    public
        constructor Create();
        destructor Destroy(); override;
        procedure Prepare();
    public
        property ChainCounter: Integer read FChainCounter;
        property Level: Integer read FLevel;
        property Threshold: Integer read FThreshold;
        property LevelTimer: Integer read FLevelTimer;
        property Timer: Integer read FTimer;
        property Score: Integer read FScore;
        property LevelUpped: Boolean read FLevelUpped;
        property BonusKind: Integer read FBonusKind;
        property GameOvered: Boolean read FGameOvered;
        property Mode: TMode read FMode;

    protected
        procedure Execute(); override;

    public
        Field: TIntField;               //  ztB[h
        Status: TIntField;              //  ΏۃZL^tB[h
        procedure GetHint(var APoint: TPoint; var ADirction: TDirection);

    public
        KindCounter: TIntCounter;       //  ޕʃJE^ = FDiffCounter
        procedure IncCounter();         //  ޕʃJE^
    end;

const
    NULL_POINT: TPoint = (X: -1; Y: -1);
    WAIT_UNIT = 30;
    DIR_TEXT: array[TDirection] of String
            = ('-', 'LEFT', 'DOWN', 'RIGHT', 'UP');

const
    SCORE_UNIT        = 50;     //  ЂƂ̓_
    SCORE_LEVEL_BONUS = 1000;   //  xAbṽ{[iX( x LEVEL)
    TIMER_BONUS       = 300;    //  ЂƂƂ̃^C}[

var
    GameThread: TGameThread = nil;

implementation

uses
    Windows, mt19937ar, Math;

//////
//-----------------------------------------------------------------------------
//  TPoint ̈v
function ComparePoint(const A, B: TPoint): Boolean;
begin
    Result := (A.X = B.X) and (A.Y = B.Y);
end;

//-----------------------------------------------------------------------------
//  TPoint ̗אړ_
function GetNeighbor(const A: TPoint; const D: TDirection): TPoint;
begin
    Result := A;
    case D of
        dLeft   : Dec(Result.X);
        dDown : Inc(Result.Y);
        dRight  : Inc(Result.X);
        dUp    : Dec(Result.Y);
    end;
end;

//////
//-----------------------------------------------------------------------------
//  RXgN^
constructor TGameThread.Create();
begin
    inherited Create(True);
    Prepare();
end;

//-----------------------------------------------------------------------------
//  
procedure TGameThread.Prepare();
begin
    if (not Suspended) then Suspend();
    ClearSelect();
    FMode := mIdle;
    FChainCounter := 0;
    InitField();
    FLevel := 0;
    NextLevel();
    FScore := 0;
    FGameOvered := False;
end;

//-----------------------------------------------------------------------------
//  fXgN^
destructor TGameThread.Destroy();
begin
    inherited;
end;

//-----------------------------------------------------------------------------
//  ĕ`v
procedure TGameThread.RepaintForm();
begin
    Application.MainForm.Repaint();
end;

//////
//-----------------------------------------------------------------------------
//  IZNA
procedure TGameThread.ClearSelect();
begin
    FSelect := NULL_POINT;
    FNeighbor := dNone;
end;

//-----------------------------------------------------------------------------
//  IZ̏㉺ẼZǂ
function TGameThread.CheckNeihbor(const APoint: TPoint): Boolean;
var
    LX: Integer;
    LY: Integer;
begin
    LX := FSelect.X - APoint.X;
    LY := FSelect.Y - APoint.Y;
    if (LX = +1) and (LY =  0) then FNeighbor := dLeft
    else
    if (LX =  0) and (LY = -1) then FNeighbor := dDown
    else
    if (LX = -1) and (LY =  0) then FNeighbor := dRight
    else
    if (LX =  0) and (LY = +1) then FNeighbor := dUp
    else
    FNeighbor := dNone;

    Result := (FNeighbor <> dNone);
end;

//-----------------------------------------------------------------------------
//  [U[̃L[
procedure TGameThread.SetPoint(const APoint: TPoint);
begin
    //  ͂ȂL^
    if (ComparePoint(FSelect, NULL_POINT)) then
    begin
        FSelect := APoint;
    end
    //  ȂNA
    else
    if (ComparePoint(FSelect, APoint)) then
    begin
        ClearSelect();
    end
    //  قȂȂאڃ`FbNאڂĂȂ΂I
    else
    begin
        if (not CheckNeihbor(APoint)) then
        begin
            ClearSelect();
            //  2007/06/23  قȂꍇ͑INAقR?
            //SetPoint(APoint);
        end;
    end;
end;

///////
//-----------------------------------------------------------------------------
//  xAbv
procedure TGameThread.NextLevel();
type
    TLevelArray = array[0..10] of Integer;
const
    NEXT_LEVEL: TLevelArray = ( 5,  7,  9, 11, 13, 15, 17, 19, 21, 23, 25);
    NEXT_TIMER: TLevelArray = (60, 60, 60, 60, 60, 60, 60, 60, 60, 60, 60);
begin
    //  ̃xŔzƁÃxł̃NA|Cgo
    if (FLevel <= High(TLevelArray)) then
    begin
        FThreshold := NEXT_LEVEL[FLevel];
        FLevelTimer := NEXT_TIMER[FLevel];
    end
    else
    begin
        FThreshold := NEXT_LEVEL[High(TLevelArray)];
        FLevelTimer := NEXT_TIMER[High(TLevelArray)];
    end;

    //  ^C}[~b
    FLevelTimer := FLevelTimer * 1000;
    FTimer := FLevelTimer;

    //  ޕʃJE^NA
    FillChar(KindCounter, SizeOf(KindCounter), 0);

    //  {[iX
    FBonusKind := (GenRandInt32() mod KIND_NUM) + 1;

    //  Ƃ̂𖈉肽
    Inc(FLevel);

    //  xAbv?
    FLevelUpped := (FLevel > 1);

    //  xAbv{[iX
    if (FLevelUpped) then
    begin
        Inc(FScore, Level * SCORE_LEVEL_BONUS);
        Inc(FScore, FTimer * SCORE_UNIT div 1000);
    end;
end;

//////
//-----------------------------------------------------------------------------
//  tB[hNA
procedure TGameThread.InitField();
var
    i, j: Integer;
    LDummy1: TPoint;
    LDummy2: TDirection;
begin
    //  ̎ύX
    InitGenRand(GetTickCount());

    //  ŃZ𖄂߂
    FillChar(Field, SizeOf(Field), NULL_CELL);
    for i := Low(Field) to High(Field) do
    begin
        for j := Low(Field[i]) to High(Field[i]) do
        begin
            Field[i][j] := (GenRandInt32() mod KIND_NUM) + 1;
        end;
    end;

    //  ŏłĂ܂Ăgݍ킹Ă
    while (Check()) do
    begin
        Delete();
        while (Drop()) do ;
    end;

    //  s\(Ȃ񂩖[v)
    if (CheckNoMove(LDummy1, LDummy2)) then
    begin
        InitField();
    end;

    //  ƗpzNA
    FillChar(Status, SizeOf(Status), STATUS_NONE);
end;

//////
//-----------------------------------------------------------------------------
//  u҂v̏
procedure TGameThread.OnIdle();
begin
    //  IZƗאڃZw肳Ă[h
    if (FNeighbor <> dNone) then
    begin
        FMode := mChange;
    end;
end;

//-----------------------------------------------------------------------------
//  uv
procedure TGameThread.OnChange();
var
    LTarget: TPoint;
    LBackValue: Integer;
begin
    //  Iꂽ̃Zւ
    if (FNeighbor <> dNone) then
    begin
        //  ւ̃Z
        LTarget := GetNeighbor(FSelect, FNeighbor);

        //  obNAbv
        LBackValue := Field[FSelect.X][FSelect.Y];

        //  ւ
        Field[FSelect.X][FSelect.Y] := Field[LTarget.X][LTarget.Y];
        Field[LTarget.X][LTarget.Y] := LBackValue;
    end;

    //  `FbN̑OƂĂ̓ւȂu`FbNvɑJ
    if (FMode = mChange) then
    begin
        FMode := mCheckViaChange;
    end
    //  `FbNs̊߂ȂIāu҂vɑJ
    else
    if (FMode = mBack) then
    begin
        ClearSelect();
        FMode := mIdle;
    end
    //  ȊO͂Ȃ if ɂ else Kv
    else
    begin
        FMode := mIdle;
    end;
end;

//-----------------------------------------------------------------------------
//  u`FbNv(`FbN)
function TGameThread.Check(): Boolean;
var
    i, j, k: Integer;
    LValue: Integer;
    LCounter: Integer;
begin
    //  ʓI1Zbgłł邩ǂ
    Result := False;

    //  \ZL^zNA
    FillChar(Status, SizeOf(Status), STATUS_NONE);

    //  ̃`FbNs

    //  ztB[hƓTCỸ`FbNσtOێzNA
    FillChar(Flag, SizeOf(Flag), False);

    //  ztB[h[瑍
    for i := Low(Field) to High(Field) do
    begin
        for j := Low(Field[i]) to High(Field[i]) do
        begin
            //  łɃ`FbNς݂̃ZȂX[
            if (Flag[i][j]) then Continue;

            //  tOĂ
            Flag[i][j] := True;

            //  ̈ʒu̒lL^(Ɠǂ`FbNĂ)
            LValue := Field[i][j];

            //  AJEgJn(g𐔂Ă)
            LCounter := 1;

            //  Ȃ̂ŏɉEӎOK
            for k := j + 1 to High(Field[i]) do
            begin
                //  Ⴄ̂ɂȂI
                if (Field[i][k] <> LValue) then Break;
                //  ȂJE^Abv
                Inc(LCounter);
                //  tOĂ
                Flag[i][k] := True;
            end;

            //  3ȏȂ
            if (LCounter >= 3) then
            begin
                Result := True;

                //  łZ̈ʒuL^
                for k := j to j + LCounter - 1 do
                begin
                    Status[i][k] := STATUS_DELETE;
                end;
            end;
        end;
    end;

    //  c̃`FbNs

    //  ztB[hƓTCỸ`FbNσtOێzNA
    FillChar(Flag, SizeOf(Flag), False);

    //  ztB[h[瑍
    for i := Low(Field) to High(Field) do
    begin
        for j := Low(Field[i]) to High(Field[i]) do
        begin
            //  łɃ`FbNς݂̃ZȂX[
            if (Flag[i][j]) then Continue;

            //  tOĂ
            Flag[i][j] := True;

            //  ̈ʒu̒lL^(Ɠǂ`FbNĂ)
            LValue := Field[i][j];

            //  AJEgJn(g𐔂Ă)
            LCounter := 1;

            //  Ȃ̂ŏɉӎOK
            for k := i + 1 to High(Field) do
            begin
                //  Ⴄ̂ɂȂI
                if (Field[k][j] <> LValue) then Break;
                //  ȂJE^Abv
                Inc(LCounter);
                //  tOĂ
                Flag[k][j] := True;
            end;

            //  3ȏȂ
            if (LCounter >= 3) then
            begin
                Result := True;

                //  łZ̈ʒuL^
                for k := i to i + LCounter - 1 do
                begin
                    Status[k][j] := STATUS_DELETE;
                end;
            end;
        end;
    end;
end;

//-----------------------------------------------------------------------------
//  u`FbNv(`FbN)
procedure TGameThread.OnCheck();
begin
    //  \ǂ`FbNB͂܂ȂB
    if (Check()) then
    begin
        //  `FbNJn͘AJE^NAAȍ~̓CNg
        if (FMode = mCheckViaChange) then
            FChainCounter := 0
        else
            Inc(FChainCounter);

        //  uvɑJ
        FMode := mDelete;

        //  INA
        ClearSelect();
    end
    //  łȂ
    else
    begin
        //  AJE^NA
        FChainCounter := 0;

        //  uv`FbN
        if (FMode = mCheckViaChange) then
        begin
            FMode := mBack;
        end
        else
        //  uAv`FbN
        if (FMode = mCheck) then
        begin
            //  x`FbN
            CheckLevel();

            FMode := mCheckNoMove;
        end
        //  ȊO͂Ȃ
        else
        begin
            FMode := mIdle;
        end;
    end;
end;

//-----------------------------------------------------------------------------
//  x
procedure TGameThread.CheckLevel();
begin
    //  SރNA|Cg𒴂Ă΃xAbv
    if (MinIntValue(KindCounter) >= FThreshold) then
    begin
        NextLevel();
    end;
end;

//-----------------------------------------------------------------------------
//  uv
procedure TGameThread.Delete();
var
    i, j: Integer;
begin
    //  JE^NA
    FillChar(FDiffCounter, SizeOf(FDiffCounter), 0);

    //  OɃ`FbNӏT
    for i := Low(Status) to High(Status) do
    begin
        for j := Low(Status[i]) to High(Status[i]) do
        begin
            if (Status[i][j] in [STATUS_DELETE, STATUS_NOMOVE]) then
            begin
                //  ޕʂɏJEg
                Inc(FDiffCounter[Field[i][j]]);
                //  
                Field[i][j] := NULL_CELL;
                //  uԂ̊Go߂̃Xe[^XύX
                Status[i][j] := STATUS_PONG;
            end;
        end;
    end;
end;

//-----------------------------------------------------------------------------
//  uv
procedure TGameThread.OnDelete();
begin
    //  猄Ԃ𖄂߂邽߂Ɂuv[h
    Delete();

    //  JE^
    IncCounter();

    FMode := mDropFirst;
end;

//-----------------------------------------------------------------------------
//  uv
function TGameThread.Drop(): Boolean;
var
    i, j: Integer;
    LCounter: Integer;
    LLowIndex: Integer;
begin
    //  Xe[^XzNA
    FillChar(Status, SizeOf(Status), STATUS_NONE);

    //  Ƃ𐔂
    LCounter := 0;

    //  iƂ(ԉ̒i͗悤Ȃ)
    for i := High(Field) - 1 downto Low(Field) do
    begin
        for j := Low(Field[i]) to High(Field[i]) do
        begin
            //  (hbv쐬̂)ۂȂ痎Ȃ
            //if (FieldGame[i][j] = NULL_CELL) then Continue;

            //  ЂƂۂȂ痎
            if (Field[i + 1][j] = NULL_CELL) then
            begin
                Field[i + 1][j] := Field[i][j];
                Field[i][j] := NULL_CELL;
                Inc(LCounter);
            end;
        end;
    end;

    //  ŏiۂɂȂ⊮
    LLowIndex := Low(Field);
    for i := Low(Field[LLowIndex]) to High(Field[LLowIndex]) do
    begin
        if (Field[LLowIndex][i] = NULL_CELL) then
        begin
            Field[LLowIndex][i] := (GenRandInt32() mod KIND_NUM) + 1;
            Inc(LCounter);
        end;
    end;

    //  ЂƂłĂяo
    Result := (LCounter > 0);

end;

//-----------------------------------------------------------------------------
//  uv
procedure TGameThread.OnDrop();
begin
    //  mDrop  mDropFirst ͑҂Ԑ̂߂ɕĂ
    if (Drop()) then
    begin
        FMode := mDrop;
    end
    //  ƂȂȂuA`FbNv
    else
    begin
        FMode := mCheck;
    end;
end;

//-----------------------------------------------------------------------------
//  uss\`FbNv
//  ss\ True / s\ False
function TGameThread.CheckNoMove(var APoint: TPoint; var ADirction: TDirection): Boolean;
var
    LCheck: TIntField;
    i, j: Integer;
    LTemp: TPoint;

    //  Ê݃`FbN
    function CheckRightOnly(): Boolean;
    var
        i, j: Integer;
        LValue: Integer;
    begin
        Result := True;

        //  ztB[h[瑍
        for i := Low(LCheck) to High(LCheck) do
        begin
            for j := Low(LCheck[i]) to High(LCheck[i]) do
            begin
                //  ̏ꏊ̒l
                LValue := LCheck[i][j];
                //  ʒuL^
                APoint := Point(i, j);

                //  
                if (j <= High(LCheck[i]) - 3) then
                begin
                    Result := (LValue <> LCheck[i][j + 2])
                           or (LValue <> LCheck[i][j + 3]);
                    if (not Result) then
                    begin
                        ADirction := dRight;
                        Exit;
                    end;
                end;

                //  ~~
                //  
                if ((i < High(LCheck)) and (j <= High(LCheck[i]) - 2)) then
                begin
                    Result := (LValue <> LCheck[i + 1][j + 1])
                           or (LValue <> LCheck[i + 1][j + 2]);
                    if (not Result) then
                    begin
                        ADirction := dDown;
                        Exit;
                    end;
                end;

                //  
                //  ~~
                if ((Low(LCheck) < i) and (j <= High(LCheck[i]) - 2)) then
                begin
                    Result := (LValue <> LCheck[i - 1][j + 1])
                           or (LValue <> LCheck[i - 1][j + 2]);
                    if (not Result) then
                    begin
                        ADirction := dUp;
                        Exit;
                    end;
                end;

                //  ~
                //  
                //  ~
                if ((Low(LCheck) < i)
                    and (i < High(LCheck))
                    and (j < High(LCheck[i]))) then
                begin
                    Result := (LValue <> LCheck[i - 1][j + 1])
                           or (LValue <> LCheck[i + 1][j + 1]);
                    if (not Result) then
                    begin
                        ADirction := dRight;
                        Exit;
                    end;
                end;
            end;
        end;
    end;

    //  `z90x]
    procedure Rotate90();
    var
        i, j: Integer;
        LTemp: TIntField;
    begin
        for i := Low(LCheck) to High(LCheck) do
        begin
            for j := Low(LCheck[i]) to High(LCheck[i]) do
            begin
                LTemp[j][High(LCheck) - i] := LCheck[i][j];
            end;
        end;
        Move(LTemp, LCheck, SizeOf(TIntField));
    end;

const
    MOD_ROTATE090: array[TDirection] of TDirection = (dNone, dDown, dRight, dUp, dLeft);
    MOD_ROTATE180: array[TDirection] of TDirection = (dNone, dRight, dUp, dLeft, dDown);
    MOD_ROTATE270: array[TDirection] of TDirection = (dNone, dUp, dLeft, dDown, dRight);
begin
    //  Xe[^XzNA
    FillChar(Status, SizeOf(Status), STATUS_NONE);

    //  Rs[
    Move(Field, LCheck, SizeOf(TIntField));

    //  Ễ݂`FbNōς܂邽߂ 90 x]Ă
    Result := CheckRightOnly();
    if (not Result) then
    begin
        Exit;
    end;

    //  90x]
    Rotate90();
    Result := CheckRightOnly();
    if (not Result) then
    begin
        LTemp := APoint;
        APoint.X := Abs(High(TIntField) - LTemp.Y);
        APoint.Y := LTemp.X;
        ADirction := MOD_ROTATE090[ADirction];
        Exit;
    end;

    //  90x](v180x])
    Rotate90();
    Result := CheckRightOnly();
    if (not Result) then
    begin
        LTemp := APoint;
        APoint.X := Abs(High(TIntField) - LTemp.X);
        APoint.Y := Abs(High(TIntField) - LTemp.Y);
        ADirction := MOD_ROTATE180[ADirction];
        Exit;
    end;

    //  90x](v270x])
    Rotate90();
    Result := CheckRightOnly();
    if (not Result) then
    begin
        LTemp := APoint;
        APoint.X := LTemp.Y;
        APoint.Y := Abs(High(TIntField) - LTemp.X);
        ADirction := MOD_ROTATE270[ADirction];
        Exit;
    end;

    //  Xe[^XzS폜ɃZbg
    for i := Low(Status) to High(Status) do
    begin
        for j := Low(Status[i]) to High(Status[i]) do
        begin
           Status[i][j] := STATUS_NOMOVE;
        end;
    end;

    //  Ȃ
    Result := True;
end;

//-----------------------------------------------------------------------------
//  ւ̂߂̃tO
procedure TGameThread.ClearGrid();
var
    i, j: Integer;
begin
    //  Xe[^XzS폜ɃZbg
    for i := Low(Status) to High(Status) do
    begin
        for j := Low(Status[i]) to High(Status[i]) do
        begin
           Status[i][j] := STATUS_NOMOVE;
        end;
    end;
end;

//-----------------------------------------------------------------------------
//  uss\`FbNv
procedure TGameThread.OnCheckNoMove();
var
    LDummy1: TPoint;
    LDummy2: TDirection;
begin
    //  ss\
    if (CheckNoMove(LDummy1, LDummy2)) then
    begin
        FMode := mNoMove;
    end
    //  s\
    else
    begin
        FMode := mIdle;
    end;
end;

//-----------------------------------------------------------------------------
//  uss\Ȃ̂őցv
procedure TGameThread.OnNoMove();
begin
    //  
    Delete();

    //  ւ
    FMode := mNoMoved;
end;

//-----------------------------------------------------------------------------
//  uւv
procedure TGameThread.AfterNoMove();
begin
    //  
    InitField();

    //  Q[ĊJ
    FMode := mIdle;
end;

//-----------------------------------------------------------------------------
//  qg
procedure TGameThread.GetHint(var APoint: TPoint; var ADirction: TDirection);
begin
    CheckNoMove(APoint, ADirction);
end;

//-----------------------------------------------------------------------------
//  ޕʃJE^݂ƃ{[iX^Cǉ
procedure TGameThread.IncCounter();
var
    i: Integer;
begin
    for i := Low(TIntCounter) to High(TIntCounter) do
    begin
        Inc(KindCounter[i], FDiffCounter[i]);
        //  ^C{[iX
        Inc(FTimer, FDiffCounter[i] * TIMER_BONUS * (FChainCounter + 1));
        //  XRA
        Inc(FScore, FLevel * SCORE_UNIT * (FChainCounter + 1));
    end;
end;

//////
//-----------------------------------------------------------------------------
//  Xbh{
procedure TGameThread.Execute();
var
    LTick: Cardinal;
    LDummy1: TPoint;
    LDummy2: TDirection;
begin
    inherited;

    while not Terminated do
    begin
        LTick := GetTickCount();

        //  󋵂Ƃɏ
        case FMode of

            mIdle           : OnIdle();
            mChange         : OnChange();
            mCheckViaChange : OnCheck();
            mBack           : OnChange();
            mDelete         : OnDelete();
            mDropFirst      : OnDrop();
            mDrop           : OnDrop();
            mCheck          : OnCheck();
            mCheckNoMove    : OnCheckNoMove();
            mNoMove         : OnNoMove();
            mNoMoved        : AfterNoMove();

        end;

        //  Q[I[o[
        FGameOvered := (FTimer < 0);

        //  xAbv~(̒͂Ȃ薳)
        if (FLevelUpped) then
        begin
            //  ʑ̏Ƃ߂
            FLevelUpped := False;

            //  xAbv
            //ClearGrid();
            //Synchronize(RepaintForm);
            //Sleep(WAIT_UNIT * 10);
            //Delete();
            //Synchronize(RepaintForm);
            //Sleep(WAIT_UNIT * 10);
            //InitField();
            //Sleep(WAIT_UNIT * 10);

            //  ʌʂł܂Ƃ߂Ďs
            repeat
                //  NA
                ClearGrid();
                //  ĕ`
                Synchronize(RepaintForm);
                //  ҂
                Sleep(WAIT_UNIT * 10);
                //  폜
                Delete();
                //  ĕ`
                Synchronize(RepaintForm);
                //  Ƃ
                while (Drop()) do
                begin
                    Sleep(WAIT_UNIT);
                    Synchronize(RepaintForm);
                end;
                //  łĂ܂Ăgݍ킹
                while (Check()) do
                begin
                    Delete();
                    Sleep(WAIT_UNIT);
                    Synchronize(RepaintForm);
                    while (Drop()) do
                    begin
                        Sleep(WAIT_UNIT);
                        Synchronize(RepaintForm);
                    end;
                end;
            until
                //  s\ɂȂ܂ł蒼
                (not CheckNoMove(LDummy1, LDummy2));

            //  ʑ̏𗬂
            FLevelUpped := True;
            Synchronize(RepaintForm);

            //  ߂
            FLevelUpped := False;
            Suspend();
        end

        //  Q[I[o[
        else if (FGameOvered) then
        begin
            //  ʑ̏Ƃ߂
            FGameOvered := False;

            //  ʌʂł܂Ƃ߂Ďs
            ClearGrid();
            Synchronize(RepaintForm);
            Sleep(WAIT_UNIT * 10);
            Delete();
            Synchronize(RepaintForm);
            Sleep(WAIT_UNIT * 10);
            InitField();
            Sleep(WAIT_UNIT * 10);

            //  ʑ̏𗬂
            FGameOvered := True;
            Synchronize(RepaintForm);

            //  ߂
            FGameOvered := False;
            Suspend();
        end

        //  ʍXV
        else
        begin
            Synchronize(RepaintForm);

            //  Ȃ̃EFCg(̓^C}[Oŏ)
            case FMode of

                mIdle           : Sleep(1);
                mChange         : Sleep(1);
                mCheckViaChange : Sleep(1);
                mBack           : Sleep(1);
                mDelete         : Sleep(WAIT_UNIT * 8);
                mDropFirst      : Sleep(WAIT_UNIT * 1);
                mDrop           : Sleep(WAIT_UNIT * 1);
                mCheck          : Sleep(WAIT_UNIT * 1);
                mCheckNoMove    : Sleep(WAIT_UNIT * 1);
                mNoMove         : Sleep(WAIT_UNIT * 10);
                mNoMoved        : Sleep(WAIT_UNIT * 5);

            end;

            //  ^C}[i߂
            Dec(FTimer, (GetTickCount() - LTick));
        end;
    end;
end;

end.


