(*
    ===========================================================================
    mt19937ar.c

    A C-program for MT19937, with initialization improved 2002/1/26.
    Coded by Takuji Nishimura and Makoto Matsumoto.

    Before using, initialize the state by using init_genrand(seed)
    or init_by_array(init_key, key_length).

    Copyright (C) 1997 - 2002, Makoto Matsumoto and Takuji Nishimura,
    All rights reserved.

    Redistribution and use in source and binary forms, with or without
    modification, are permitted provided that the following conditions
    are met:

      1. Redistributions of source code must retain the above copyright
         notice, this list of conditions and the following disclaimer.

      2. Redistributions in binary form must reproduce the above copyright
         notice, this list of conditions and the following disclaimer in the
         documentation and/or other materials provided with the distribution.

      3. The names of its contributors may not be used to endorse or promote
         products derived from this software without specific prior written
         permission.

    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
    "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
    LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
    A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
    CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
    EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
    PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
    PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
    LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
    NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
    SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


    Any feedback is very welcome.
    http://www.math.sci.hiroshima-u.ac.jp/~m-mat/MT/emt.html
    email: m-mat @ math.sci.hiroshima-u.ac.jp (remove space)
    ===========================================================================
*)

(*
    ===========================================================================
    mt19937ar.c̎g i2002/1, 2004/2j

    ̃R[hMersenne Twister[@łB2^19937-1ŁA623
    ϓzۏႳ܂B2002N1ɉǂ܂B

    F
    ̏[`Iׂ܂Bǂ炩́AgpOɂȂ炸ĂяoȂ
    Ȃ܂B

    init_genrand(seed)ł́ACӂ̕Ȃꃏ[hV[hƂė^
    邱Ƃł܂i0ł܂܂jB

    init_by_array(init_key, key_length)ł́AȂ[h̔zinit_key
    i[ꂽV[hƂė^邱Ƃł܂B
    key_length́A̔z̓牽[hڂ܂łV[hƂ邩w
    ܂Bz̒́A1ȏCӂłB

    F
    L̓֐̂ꂩpďÃ^Cv̋[gpł
    ܂B

        genrand_int32() Ȃ32rbg
        genrand_int31() Ȃ31rbg
        genrand_real1() l[0,1] (32rbgx).
        genrand_real2() l[0,1) (32rbgx).
        genrand_real3() l(0,1) (32rbgx).
        genrand_res53() l[0,1) (53rbgx).

    ŁA[0,1]0,1܂ށA[0,1)0͊܂ނ1͊܂܂ȂAȂǂ\܂B

    main()F
    vOłBinit_by_arrayɂAS̔z sĂ܂B
    Ȃ32rbg1000A[0,1)l 1000o͂܂B

    o:
    ̃vȌo͂Amt19937ar.outłBR[hύXʂ̌Ɉ
    AꍇȀo͂păeXg邱Ƃ߂܂B
    ===========================================================================
*)

{
//-----------------------------------------------------------------------------
//  These real versions are due to Isaku Wada, 2002/01/09 added
procedure TForm1.Button1Click(Sender: TObject);
var
    i: Integer;
    c: Cardinal;
    r: double;
    FileName: String;
    FileStream: TFileStream;
    Line: String;
begin

    FileName := ChangeFileExt(ParamStr(0), '.csv');
    FileStream := TFileStream.Create(
                        FileName,
                        fmCreate or fmShareDenyWrite);
    for i := 0 to 999 do
    begin
        c := GenRandInt32();
        Line := Format('%10u ', [c]);
        FileStream.Write(Line[1], Length(Line));
        if (i mod 5 = 4) then
        begin
            Line := #13#10;
            FileStream.Write(Line[1], Length(Line));
        end;
    end;
    for i := 0 to 999 do
    begin
        r := GenRandReal2();
        Line := Format('%10.8f ', [r]);
        FileStream.Write(Line[1], Length(Line));
        if (i mod 5 = 4) then
        begin
            Line := #13#10;
            FileStream.Write(Line[1], Length(Line));
        end;
    end;
    FileStream.Free;

    Application.MessageBox(PChar(FileName + 'ɏo͊'), PChar(''),
                MB_OK or MB_ICONINFORMATION);
end;
}

//  2005.12.27  BLUELEAF1336 |

unit mt19937ar;
{$OVERFLOWCHECKS OFF}
{$RANGECHECKS ON}
interface

procedure InitGenRand(const Seed: Cardinal);
function GenRandInt32(): Cardinal;
function GenRandInt31(): Integer;
function GenRandReal1(): Double;
function GenRandReal2(): Double;
function GenRandReal3(): Double;
function GenRandRes53(): Double;

implementation

//  Period parameters
const
    MERSENNE_TWISTER_N = 624;
    MERSENNE_TWISTER_M = 397;
    MERSENNE_TWISTER_MATRIX_A   = $9908b0df;    //  constant vector a
    MERSENNE_TWISTER_UPPER_MASK = $80000000;    //  most significant w-r bits
    MERSENNE_TWISTER_LOWER_MASK = $7fffffff;    //  least significant r bits
    F_MASK = $FFFFFFFF;

var
    //  the array for the state vector (unsigned long)
    Mt: array[0..MERSENNE_TWISTER_N-1] of Cardinal;

    //  mti==N+1 means mt[N] is not initialized
    Mti: Cardinal = MERSENNE_TWISTER_N + 1;

//-----------------------------------------------------------------------------
//  initializes mt[N] with a seed (unsigned long)
procedure InitGenRand(const Seed: Cardinal);
begin
    Mt[0] := Seed and F_MASK;

    Mti := 1;
    while (Mti < MERSENNE_TWISTER_N) do
    begin
        Mt[Mti] := (1812433253 * (Mt[Mti-1] xor (Mt[Mti-1] shr 30)) + Mti);
        //  See Knuth TAOCP Vol2. 3rd Ed. P.106 for multiplier.
        //  In the previous versions, MSBs of the seed affect
        //  only MSBs of the array mt[].
        //  2002/01/09 modified by Makoto Matsumoto

        Mt[Mti] := Mt[Mti] and F_MASK;
        //  for >32 bit machines

        Inc(Mti);
    end;
end;

//-----------------------------------------------------------------------------
function Max(const A, B: Integer): Integer;
begin
    if (A >= B) then Result := A else Result := B;
end;

//-----------------------------------------------------------------------------
//  initialize by an array with array-length
//  init_key is the array for initializing keys
//  key_length is its length
//  slight change for C++, 2004/2/26
procedure InitByArray(const Seeds: array of Cardinal);
var
    i, j, k: Cardinal;
    SeedsLen: Cardinal;
begin
    InitGenRand(19650218);

    i := 1;
    j := 0;

    SeedsLen := Length(Seeds);
    k := Max(SeedsLen, MERSENNE_TWISTER_N);

    while (k > 0) do
    begin
        Mt[i] := (Mt[i] xor ((Mt[i-1] xor (Mt[i-1] shr 30)) * 1664525))
               + Seeds[j] + j;          //  non linear
        Mt[i] := Mt[i] and F_MASK;      //  for WORDSIZE > 32 machines

        Inc(i);
        Inc(j);

        if (i >= MERSENNE_TWISTER_N) then
        begin
            Mt[0] := Mt[MERSENNE_TWISTER_N-1];
            i := 1;
        end;

        if (j >= SeedsLen) then
        begin
            j := 0;
        end;

        Dec(k);
    end;

    for k := MERSENNE_TWISTER_N - 1 downto 1 do
    begin
        Mt[i] := (Mt[i] xor ((Mt[i-1] xor (Mt[i-1] shr 30)) * 1566083941))
               - i;                     //  non linear
        Mt[i] := Mt[i] and F_MASK;      //  for WORDSIZE > 32 machines
        Inc(i);

        if (i >= MERSENNE_TWISTER_N) then
        begin
            Mt[0] := Mt[MERSENNE_TWISTER_N-1];
            i := 1;
        end;
    end;

    //  MSB is 1; assuring non-zero initial array
    Mt[0] := $80000000;
end;

//-----------------------------------------------------------------------------
//  generates a random number on [0,0xffffffff]-interval
function GenRandInt32(): Cardinal;
const
    //  mag01[x] = x * MATRIX_A  for x=0,1
    MAG01: array[0..1] of Cardinal = (0, MERSENNE_TWISTER_MATRIX_A);
var
    kk: Integer;
begin
    //  generate N words at one time
    if (Mti >= MERSENNE_TWISTER_N) then
    begin
        //  if init_genrand() has not been called,
        if (Mti = MERSENNE_TWISTER_N + 1) then
        begin
            //  a default initial seed is used
            InitGenRand(5489);
        end;

        kk := 0;
        while (kk < MERSENNE_TWISTER_N - MERSENNE_TWISTER_M) do
        begin
            Result := (Mt[kk] and MERSENNE_TWISTER_UPPER_MASK)
                   or (Mt[kk+1] and MERSENNE_TWISTER_LOWER_MASK);

            Mt[kk] := Mt[kk + MERSENNE_TWISTER_M]
                  xor (Result shr 1)
                  xor MAG01[Result and 1];

            Inc(kk);
        end;

        while (kk < MERSENNE_TWISTER_N - 1) do
        begin
            Result := (Mt[kk] and MERSENNE_TWISTER_UPPER_MASK)
                   or (Mt[kk+1] and MERSENNE_TWISTER_LOWER_MASK);

            Mt[kk] := Mt[kk + (MERSENNE_TWISTER_M - MERSENNE_TWISTER_N)]
                  xor (Result shr 1)
                  xor MAG01[Result and 1];

            Inc(kk);
        end;

        Result := (Mt[MERSENNE_TWISTER_N-1] and MERSENNE_TWISTER_UPPER_MASK)
               or (Mt[0] and MERSENNE_TWISTER_LOWER_MASK);

        Mt[MERSENNE_TWISTER_N-1] := Mt[MERSENNE_TWISTER_M-1]
                                  xor (Result shr 1)
                                  xor MAG01[Result and 1];

        Mti := 0;
    end;

    Result := Mt[Mti];
    Inc(Mti);

    //  Tempering
    Result := Result xor  (Result shr 11);
    Result := Result xor ((Result shl  7) and $9d2c5680);
    Result := Result xor ((Result shl 15) and $efc60000);
    Result := Result xor  (Result shr 18);
end;

//-----------------------------------------------------------------------------
//  generates a random number on [0,0x7fffffff]-interval
function GenRandInt31(): Integer;
begin
    Result := (GenRandInt32() shr 1);
end;

//-----------------------------------------------------------------------------
//  generates a random number on [0,1]-real-interval
function GenRandReal1(): Double;
begin
    //  divided by 2^32-1
    Result := (GenRandInt32() * (1.0 / 4294967295.0));
end;

//-----------------------------------------------------------------------------
//  generates a random number on [0,1)-real-interval
function GenRandReal2(): Double;
begin
    //  divided by 2^32
    Result := (GenRandInt32() * (1.0 / 4294967296.0));
end;

//-----------------------------------------------------------------------------
//  generates a random number on (0,1)-real-interval
function GenRandReal3(): Double;
begin
    //  divided by 2^32
    Result := (GenRandInt32() + 0.5) * (1.0 / 4294967296.0);
end;

//-----------------------------------------------------------------------------
//  generates a random number on [0,1) with 53-bit resolution
function GenRandRes53(): Double;
var
    a, b: Cardinal;
begin
    a := GenRandInt32() shr 5;
    b := GenRandInt32() shr 6;
    Result := (a * 67108864.0 + b) * (1.0 / 9007199254740992.0);
end;

//-----------------------------------------------------------------------------
const
    DefaultSeeds: array[0..3] of Cardinal = ($123, $234, $345, $456);
initialization
    //InitGenRand(19650218);
    InitByArray(DefaultSeeds);
end.



